@@ -1,7 +1,8 @@
-### Copyright (C) 2001-2005  Deepayan Sarkar <Deepayan.Sarkar@R-project.org>
+
+### Copyright (C) 2001-2006  Deepayan Sarkar <Deepayan.Sarkar@R-project.org>
 ### Copyright (C) 2001-2005  Saikat DebRoy <saikat@stat.wisc.edu>
 ###
-### This file is part of the lattice library for R.
+### This file is part of the lattice package for R.
 ### It is made available under the terms of the GNU General Public
 ### License, version 2, or at your option, any later version,
 ### incorporated herein by reference.
@@ -14,32 +15,29 @@
 ###
 ### You should have received a copy of the GNU General Public
 ### License along with this program; if not, write to the Free
-### Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-### MA 02111-1307, USA
+### Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+### MA 02110-1301, USA
 
 
 
 
 cupdate <- function(index, maxim)
 {
-    ##  This function is used to handle arbitrary number of
-    ## conditioning variables : every time it is called, it
-    ## increments the "current" level of the conditioning
-    ## variables suitably, i.e., it tries to increment the
-    ## level of the 1st conditining variable (the one which
-    ## varies fastest along panel order) and if it happens
-    ## to be at its maximum (last) value, it sets it to the
-    ## first value AND increments the "current" level of the
-    ## 2nd (next) conditioning variable recursively.
 
-    ## This is an internal function, not to be documented
-    ## for the high level user.
-    
+    ## This unexported function is used to handle arbitrary number of
+    ## conditioning variables : every time it is called, it increments
+    ## the "current" level of the conditioning variables suitably,
+    ## i.e., it tries to increment the level of the 1st conditining
+    ## variable (the one which varies fastest along panel order) and
+    ## if it happens to be at its maximum (last) value, it sets it to
+    ## the first value AND increments the "current" level of the 2nd
+    ## (next) conditioning variable recursively.
+
     if(length(index)!=length(maxim)||length(maxim)<=0)
         stop("Inappropriate arguments")
     index[1] <- index[1] + 1
-    if(index[1]>maxim[1] && length(maxim)>1)
-        c(1,cupdate(index[-1],maxim[-1]))
+    if (index[1] > maxim[1] && length(maxim) > 1)
+        c(1, cupdate(index[-1], maxim[-1]))
     else index
 }
 
@@ -52,11 +50,25 @@ cupdate <- function(index, maxim)
 #     names.current <- as.character(names.current)
 #     new.prefix <- as.character(new.prefix)
 #     newnames <- c(new.prefix,
-#                   paste(gvar, seq(along=names.current), sep=""))
+#                   paste(gvar, seq_along(names.current), sep=""))
 #     newnames[!(newnames %in% names.current)][1]
 # }
 
 
+## a function similar to (but with much less bells and whistles than)
+## interaction, with exactly 2 factors, and levels ordered like that
+## in f:g (the point is to have a 'sep' argument).  Solely intended
+## for use below in latticeParseFormula
+
+interaction2 <- 
+    function (f, g, sep = ":")
+{
+    ans <- f:g
+    levels(ans) <-
+        as.vector(t(outer(levels(f), levels(g),
+                          paste, sep = sep)))
+    ans
+}
 
 
 
@@ -67,13 +79,13 @@ latticeParseFormula <-
     ## this function mostly written by Saikat
 {
 
-    ## local function to get length-1 names from expressions (deparse
-    ## can be longer). Could be either of the following
+    ## local function to get length 1 name from expressions (deparse
+    ## can split long expressions into several pieces), typically for
+    ## subsequent use as labels.  Could be either of the following:
 
     expr2char <- function(x) paste(deparse(x), collapse = "")
     ## expr2char <- function(x) deparse(x)[1]
 
-
     ## by this time, groups is usually already evaluated.  To make
     ## things slightly more convenient, we will now also allow groups
     ## to be of the form groups = ~g, in which case g will be
@@ -82,7 +94,7 @@ latticeParseFormula <-
     if (inherits(groups, "formula"))
     {
         groupVar <- as.character(groups)[2]
-        groups <- eval(parse(text = groupVar), data, parent.frame())
+        groups <- eval(parse(text = groupVar), data, environment(groups))
     }
 
     if (is.null(drop)) drop <- TRUE
@@ -150,7 +162,7 @@ latticeParseFormula <-
     {
         ## ignore multiple = TRUE
         multiple <- FALSE
-        warning("multiple = TRUE ignored (groups non-null with outer = FALSE)")
+        warning("'multiple=TRUE' ignored ('groups' non-null with 'outer=FALSE')")
     }
 
     ans <- if (dimension == 2) {
@@ -162,8 +174,7 @@ latticeParseFormula <-
              left.name = character(0), right.x.name = character(0),
              right.y.name = character(0))
     }
-    else stop(paste("invalid dimension : ", dimension))
-
+    else stop(gettextf("invalid dimension '%s'", as.character(dimension)))
     
     if (length(model) == 3) {  ## <something> ~ <something>
         if (multiple) {
@@ -197,7 +208,7 @@ latticeParseFormula <-
         }
         ans$condition <- vector("list", length(modelRHS.vars))
         names(ans$condition) <- sapply(modelRHS.vars, expr2char)
-        for (i in seq(along = modelRHS.vars)) {
+        for (i in seq_along(modelRHS.vars)) {
             ans$condition[[i]] <-
                 lrep(as.factorOrShingle(eval(modelRHS.vars[[i]], data, env),
                                         subset, drop = drop.unused.cond), nLHS * nRHS)
@@ -210,7 +221,8 @@ latticeParseFormula <-
         nRHS <- 1
     }
 
-    if (length(model) == 3) {
+    if (length(model) == 3)
+    {
 
         ## Note: special case if tmp is a matrix. This probably means
         ## we are dealing with the parametric 3-D surface rendering
@@ -234,13 +246,26 @@ latticeParseFormula <-
                                })), nRHS)
     }
 
-    if (dimension == 2) {
+    if (dimension == 2)
+    {
 
         ## this part belongs in the first if block, but we need nobs
         ## later in either case, so keeping it here.
 
         tmp <- eval(varsRHS[[1]], data, env)
         if (is.matrix(tmp)) tmp <- as.data.frame(tmp)
+
+### This was a attempted new feature that turned out not to be such a
+### good idea.  It would change '~x | a' to 'rownames(data) ~ x | a',
+### which is good for dot plot, but not good for bwplot.  So, back to
+### old behaviour...
+
+##         else if (length(model) == 2 && is.data.frame(data) &&
+##                  (is.vector(tmp, "numeric") || is.factor(tmp)) &&
+##                  is.null(names(tmp)))
+##         {
+##             names(tmp) <- rownames(data) # for dotplot(~x, data)
+##         }
         nobs <- if (is.data.frame(tmp)) nrow(tmp) else length(tmp)
 
         if (nLHS == 1 && nRHS == 1) {
@@ -273,7 +298,8 @@ latticeParseFormula <-
         nRows <- length(ans$right)/(nLHS * nRHS)
     }
     else if (dimension == 3 && length(modelRHS) == 3 &&
-             (modelRHS[[1]] == "*" || modelRHS[[1]] == "+")) {
+             (modelRHS[[1]] == "*" || modelRHS[[1]] == "+"))
+    {
 
         ## Note that when dimension = 3, multiple does not apply to
         ## RHS, but nLHS may be > 1
@@ -318,7 +344,8 @@ latticeParseFormula <-
             gl(nRHS, nRows*nLHS, labels = sapply(varsRHS, expr2char))
     newFactor <- 
         if (nLHS > 1 && nRHS > 1) {
-            factor(paste(LHSgroups, RHSgroups, sep=" * "))
+            interaction2(LHSgroups, RHSgroups, sep = lattice.getOption("interaction.sep"))
+            ## WAS: factor(paste(LHSgroups, RHSgroups, sep=" * "))
         } else if (nLHS > 1)
             LHSgroups
         else if (nRHS > 1)
@@ -337,7 +364,7 @@ latticeParseFormula <-
         ## in panel functions.
 
         if (!is.null(groups)) ans$groups <- groups
-        if (subscripts) ans$subscr <- seq(length = nobs)[subset]
+        if (subscripts) ans$subscr <- seq_len(nobs)[subset]
     }
     else if (outer) {
         if (!is.null(groups)) ans$groups <- rep(groups, nLHS * nRHS)
@@ -356,7 +383,7 @@ latticeParseFormula <-
 
         if (subscripts)
             ans$subscr <-
-                as.vector(matrix(seq(length = nobs * nLHS * nRHS), nrow = nobs)[subset, ])
+                as.vector(matrix(seq_len(nobs * nLHS * nRHS), nrow = nobs)[subset, ])
     }
     else {  ## that is, nLHS * nRHS > 1, outer = FALSE
         if (is.null(groups) && !is.null(newFactor))
@@ -368,7 +395,7 @@ latticeParseFormula <-
         ## variables. So subscr has to be a simple seq, as long as the
         ## new groups.
 
-        if (subscripts) ans$subscr <- seq(length = length(newFactor))
+        if (subscripts) ans$subscr <- seq_len(length(newFactor))
 
         ## check
         if (length(newFactor) != nRows * nLHS * nRHS)
@@ -403,37 +430,40 @@ banking <- function(dx, dy = 1)
 
 
 
+## method in Visualizing Data: needs x-range and y-range as well.  Not
+## sure what the best way to extend the API is; for now, we will just
+## define (but not export) an implementation
 
 
-
-
-
-
-
-extend.limits <-
-    function(lim, length=1, axs = "r",
-             prop = if (axs == "i") 0 else lattice.getOption("axis.padding")$numeric)
+weighted.banking <-
+    function(dx, dy = 1, xrange = sum(dx), yrange = sum(dy))
 {
-    if (!is.numeric(lim)) NA
-    else if(length(lim)==2) {
-        if (lim[1]>lim[2]) stop("Improper value of limit")
-        if (!missing(length) && !missing(prop))
-            stop("length and prop cannot both be specified")
-        if (length <= 0) stop("length must be positive")
-        if (!missing(length))
+    if (is.list(dx)) {
+        if (is.null(names(dx))) names(dx) <- c("dx", "dy")
+        dy <- abs(dx[["dy"]])
+        dx <- abs(dx[["dx"]])
+    }
+    if (length(dx)!=length(dy)) stop("Non matching lengths")
+    id <- dx!=0 & dy!=0 & !is.na(dx) & !is.na(dy)
+    if (any(id))
+    {
+        ndx <- dx / xrange
+        ndy <- dy / yrange
+        obj <- function(a)
         {
-            prop <- (as.numeric(length) - as.numeric(diff(lim))) / (2 * as.numeric(diff(lim)))
-        }
-        if (lim[1]==lim[2]) lim + 0.5 * c(-length,length)
-        else {
-            d <- diff(as.numeric(lim))
-            lim + prop * d * c(-1,1)
+            sum(atan(a * ndy / ndx) * sqrt(ndx^2 + a^2 * ndy^2) ) /
+                sum(sqrt(ndx^2 + a^2 * ndy^2)) -
+                    base::pi / 4 
         }
+        ##         r  <- abs(dx[id]/dy[id])
+        ##         median(r)
+        ## transform to have arg between 0 and 1
+        objtan <- function(x) obj(tan(base::pi * x / 2))
+        u <- uniroot(f = objtan, lower = 0.01, upper = 0.99)
+        aspect.ratio <- tan(base::pi * u$root / 2)
+        aspect.ratio * xrange / yrange
     }
-    else {
-        print(lim)
-        stop("improper length of lim")
-    }
+    else 1
 }
 
 
@@ -442,10 +472,75 @@ extend.limits <-
 
 
 
+
+
+extend.limits <-
+    function(lim, length = 1, axs = "r",
+             prop =
+             if (axs == "i") 0
+             else lattice.getOption("axis.padding")$numeric)
+{
+    ## if (!is.numeric(lim)) NA
+    if (is.character(lim)) NA # or lim?
+    else if (length(lim) == 2)
+    {
+        if (lim[1] > lim[2])
+        {
+            ccall <- match.call()
+            ccall$lim <- rev(lim)
+            ans <- eval.parent(ccall)
+            return (rev(ans))
+        }
+        if (!missing(length) && !missing(prop))
+            stop("'length' and 'prop' cannot both be specified")
+        if (length <= 0) stop("'length' must be positive")
+        if (!missing(length))
+        {
+            prop <- (as.numeric(length) - as.numeric(diff(lim))) / (2 * as.numeric(diff(lim)))
+        }
+        if (lim[1]==lim[2]) lim + 0.5 * c(-length,length)
+        else
+        {
+            d <- diff(as.numeric(lim))
+            lim + prop * d * c(-1,1)
+        }
+    }
+    else
+    {
+        print(lim)
+        stop("improper length of 'lim'")
+    }
+}
+
+
+
+
+### this is a catch-common-arguments function that intercepts several
+### common arguments.  These are currently documented in ?xyplot,
+### under \dots (...).  Some arguments of trellis.skeleton are also
+### formal arguments to xyplot, and are documented as such.  These are
+### indicated below with a comment.  This is useful to keep track of
+### documented arguments, because the formal codoc system doesn't work
+### here (trellis.skeleton is not exported and hence not formally
+### documented).
+
+
+### one special exception is 'subscripts'.  A 'subscripts' argument is
+### required in the panel functions for panel.identify etc to work
+### properly.  However, not all high level functions always pass a
+### suitable 'subscripts' argument to its panel function (but some,
+### like 'splom' and 'levelplot' do).  For those that do not, a
+### 'subscripts=TRUE' argument can change the behaviour.  The others
+### don't have a 'subscripts' argument (as it is redundant), but we'll
+### capture and ignore it here to keep the interface consistent.
+
+
+
 trellis.skeleton <-
-    function(cond,
+    function(formula = NULL,
+             cond,
+             aspect = default.args$aspect, # argument in xyplot
              as.table = default.args$as.table,
-             aspect = default.args$aspect,
              between = default.args$between,
              key = NULL,
              legend = NULL,
@@ -455,42 +550,60 @@ trellis.skeleton <-
              par.strip.text = default.args$par.strip.text,
              layout = default.args$layout,
              skip = default.args$skip,
-             strip = default.args$strip.default,
-             xlab = NULL,
-             ylab = NULL,
+             strip = default.args$strip.default, # argument in xyplot
+             strip.left = FALSE,
              xlab.default = NULL,
              ylab.default = NULL,
-             panel,
-             par.settings = NULL,
+             xlab = NULL, # argument in xyplot
+             ylab = NULL, # argument in xyplot
+             panel,       # argument in xyplot
+
+             xscale.components = default.args$xscale.components,
+             yscale.components = default.args$yscale.components,
+             axis = default.args$axis,
+
+             subscripts = TRUE, # ignored, for reasons given above
+
              index.cond = NULL,
              perm.cond = NULL,
-             ...)
+             ...,
+             par.settings = NULL,
+             plot.args = NULL,
+             lattice.options = NULL)
 {
     default.args <- lattice.getOption("default.args")
-
     if (is.null(skip)) skip <- FALSE
-    foo <- list(as.table = as.table,
-                aspect.fill = aspect == "fill",
-                ## key = key,
-                legend = construct.legend(legend = legend, key = key),
-                panel = panel, 
-                page = page,
-                layout = layout,
-                skip = skip,
-                strip = if (is.logical(strip) && strip) "strip.default"
-                else strip,
-                xlab = xlab,
-                ylab = ylab,
-                xlab.default = xlab.default,
-                ylab.default = ylab.default,
-                main = main,
-                sub = sub,
-                x.between = 0,
-                y.between = 0,
-                par.settings = par.settings,
-                par.strip.text = par.strip.text,
-                index.cond = index.cond,
-                perm.cond = perm.cond)
+    foo <-
+        list(formula = formula,
+             as.table = as.table,
+             aspect.fill = (aspect == "fill"),
+             ## key = key,
+             legend = construct.legend(legend = legend, key = key),
+             panel = panel, 
+             page = page,
+             layout = layout,
+             skip = skip,
+             strip = if (is.logical(strip) && strip) "strip.default"
+             else strip,
+             strip.left = if (is.logical(strip.left) && strip.left) strip.custom(horizontal = FALSE)
+             else strip.left,
+             xscale.components = xscale.components,
+             yscale.components = yscale.components,
+             axis = axis,
+             xlab = xlab,
+             ylab = ylab,
+             xlab.default = xlab.default,
+             ylab.default = ylab.default,
+             main = main,
+             sub = sub,
+             x.between = 0,
+             y.between = 0,
+             par.settings = par.settings,
+             plot.args = plot.args,
+             lattice.options = lattice.options,
+             par.strip.text = par.strip.text,
+             index.cond = index.cond,
+             perm.cond = perm.cond)
 
     if (!is.null(between$x)) foo$x.between <- between$x
     if (!is.null(between$y)) foo$y.between <- between$y
@@ -536,9 +649,9 @@ cond.orders <- function(foo, ...)
         vector(mode = "list",
                length = length(foo$condlevels))
 
-    for (i in seq(along = foo$condlevels))
-        index.cond[[i]] <- seq(along = foo$condlevels[[i]])
-    perm.cond <- seq(length = length(foo$condlevels))
+    for (i in seq_along(foo$condlevels))
+        index.cond[[i]] <- seq_along(foo$condlevels[[i]])
+    perm.cond <- seq_len(length(foo$condlevels))
 
     if (!is.null(foo$perm.cond))
     {
@@ -550,7 +663,7 @@ cond.orders <- function(foo, ...)
     {
         if (is.list(foo$index.cond) && length(foo$index.cond) == length(index.cond))
         {
-            for (i in seq(along = foo$condlevels))
+            for (i in seq_along(foo$condlevels))
                 index.cond[[i]] <- index.cond[[i]][foo$index.cond[[i]]]
         }
         else if (is.function(foo$index.cond))
@@ -558,13 +671,13 @@ cond.orders <- function(foo, ...)
             FUN <- foo$index.cond
             nplots <- length(foo$panel.args)
             panel.order <- numeric(nplots)
-            for (count in seq(length = nplots))
+            for (count in seq_len(nplots))
             {
                 if (is.list(foo$panel.args[[count]]))
                 {
                     pargs <- c(foo$panel.args.common, foo$panel.args[[count]], list(...))
                     prenames <- names(formals(FUN))
-                    if (!("..." %in% prenames)) pargs <- pargs[prenames]
+                    if (!("..." %in% prenames)) pargs <- pargs[intersect(names(pargs), prenames)]
                     panel.order[count] <- do.call("FUN", pargs)
                 }
                 else  ## this happens for empty panels
@@ -573,7 +686,7 @@ cond.orders <- function(foo, ...)
                 }
             }
             dim(panel.order) <- sapply(foo$condlevels, length)
-            for (i in seq(along = foo$condlevels))
+            for (i in seq_along(foo$condlevels))
                 index.cond[[i]] <-
                     order(apply(panel.order, i, mean, na.rm = TRUE))
         }
@@ -599,10 +712,12 @@ compute.layout <-
     number.of.cond <- length(cond.max.level)
     nplots <- prod(cond.max.level)
     
-    if (!is.numeric(layout)) {
+    if (!is.numeric(layout))
+    {
         layout <- c(0,1,1)
-        if (number.of.cond==1) layout[2] <- nplots
-        else {
+        if (number.of.cond == 1) layout[2] <- nplots
+        else
+        {
             layout[1] <- cond.max.level[1]
             layout[2] <- cond.max.level[2]
         }
@@ -610,20 +725,21 @@ compute.layout <-
         plots.per.page <- length(skip) - length(skip[skip])
         layout[3] <- ceiling(nplots/plots.per.page) # + 1
     }
-    else if (length(layout)==1)
+    else if (length(layout) == 1)
         stop("layout must have at least 2 elements")
-    else if (length(layout)==2)
+    else if (length(layout) == 2)
     {
-        if(all(layout<1))
+        if(all(layout < 1))
             stop("at least one element of layout must be positive")
         else if (layout[2]==0) stop("inadmissible value of layout")
         
         skip <- rep(skip, length = max(layout[1] * layout[2], layout[2]))
         plots.per.page <- length(skip) - length(skip[skip])
-        layout[3] <- ceiling(nplots/plots.per.page) # + 1 
+        layout[3] <- ceiling(nplots / plots.per.page) # + 1 
     }
-    else if (length(layout)==3) {
-        if(layout[1]<0||layout[2]<1||layout[3]<1)
+    else if (length(layout)==3)
+    {
+        if(layout[1] < 0 || layout[2] < 1 || layout[3] < 1)
             stop("invalid value for layout")
     }
     layout
@@ -631,3 +747,24 @@ compute.layout <-
 
 
 
+
+compute.packet <-
+    function(cond, levels)
+{
+    id <- !(do.call("pmax", lapply(cond, is.na)))
+    stopifnot(any(id))
+    for (i in seq_along(cond))
+    {
+        var <- cond[[i]]
+        id <-
+            id & (
+                  if (is.shingle(var))
+                  ((var >= levels(var)[[levels[i]]][1]) &
+                   (var <= levels(var)[[levels[i]]][2]))
+                  else
+                  (as.numeric(var) == levels[i])
+                  )
+    }
+    id
+}
+
