@@ -2,7 +2,7 @@ c-----------------------------------------------------------------------
 c
 c  R : A Computer Langage for Statistical Data Analysis
 c  Copyright (C) 1996, 1997  Robert Gentleman and Ross Ihaka
-c  Copyright (C) 2001        The R Development Core Team
+c  Copyright (C) 2001-08     The R Development Core Team
 c
 c  This program is free software; you can redistribute it and/or modify
 c  it under the terms of the GNU General Public License as published by
@@ -15,8 +15,8 @@ c  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 c  GNU General Public License for more details.
 c
 c  You should have received a copy of the GNU General Public License
-c  along with this program; if not, write to the Free Software
-c  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+c  along with this program; if not, a copy is available at
+c  http://www.r-project.org/Licenses/
 c
 c-----------------------------------------------------------------------
 c
@@ -45,6 +45,9 @@ c		the value of inverse(x'x)
 c
 c This version dated Aug 24, 1996.
 c Ross Ihaka, University of Auckland.
+c
+c This is currently only used via R's  chol2inv(..., LINPACK = TRUE)
+c which is said to be only for compatibility with R < 1.7.0
 c
       subroutine ch2inv(x, ldx, n, v, info)
 c     implicit none
@@ -52,7 +55,7 @@ c     implicit none
       double precision x(ldx, n), v(n, n)
 c
       double precision d
-      integer i, j, im1
+      integer i, j
 c
       do 20 i=1,n
         if(x(i,i) .eq. 0.0d0) then
@@ -64,9 +67,8 @@ c
    10   continue
    20 continue
       call dpodi(v, n, n, d, 1)
-      do 40 i=1,n
-        im1 = i-1
-        do 30 j=1,im1
+      do 40 i=2,n
+        do 30 j=1,i-1
           v(i,j) = v(j,i)
    30   continue
    40 continue
