@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2001  R Development Core Team
+ *  Copyright (C) 2001-7  R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *  along with this program; if not,  a copy is available at
  *  http://www.r-project.org/Licenses/
  */
-#define NONAMELESSUNION
+
 #include <windows.h>
 #include <stdio.h>
 #include <stdlib.h>		/* for exit */
@@ -24,7 +24,7 @@
 
 #define REG_KEY_NAME "Software\\R-core\\R"
 
-extern char *getRHOME(); /* in ../rhome.c */
+extern char *getRHOMElong(void); /* in ../rhome.c */
 
 int main (int argc, char **argv)
 {
@@ -64,7 +64,7 @@ int main (int argc, char **argv)
 	    status = 1;
 	}
     } else {
-    	RHome = getRHOME();
+    	RHome = getRHOMElong();
 	if ((rc = RegOpenKeyEx(HKEY_LOCAL_MACHINE, REG_KEY_NAME, 0, 
 			       KEY_ALL_ACCESS, &hkey)) != ERROR_SUCCESS) {
 	    /* failed to open key, so try to create it */
