@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2000-4  R Development Core Team
+ *  Copyright (C) 2000-8  R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 #include <stdio.h>
 #include <Rversion.h>
 
-extern char *getRHOME(), *getRUser(); /* in ../rhome.c */
+extern char *getRHOME(void), *getRUser(void); /* in ../rhome.c */
 
 void R_Suicide(char *s) /* for use in ../rhome.o */
 {
@@ -76,8 +76,10 @@ int rcmdfn (int cmdarg, int argc, char **argv)
        set R_SHARE_DIR as env variable
        set PATH to include R_HOME\bin
        set PERL5LIB to %R_SHARE_DIR%/perl;%Perl5LIB%
-       set TEXINPUTS to %R_SHARE_DIR%/texmf;%TEXINPUTS%
+       set TEXINPUTS to .;%R_SHARE_DIR%/texmf;%TEXINPUTS%
+       set TMPDIR if unset
        set HOME if unset
+       set R_CMD (depends on how this was launched), R_VERSION, R_OSTYPE
        launch %R_HOME%\bin\$*
      */
     int i, iused, status = 0;
@@ -243,8 +245,9 @@ int rcmdfn (int cmdarg, int argc, char **argv)
 	putenv(RHOME);
 
 	/* currently used by Rd2dvi and by perl Vars.pm (with default) */
-	strcpy(RSHARE, "R_START_DIR=");
+	strcpy(RSHARE, "R_SHARE_DIR=");
 	strcat(RSHARE, RHome); strcat(RSHARE, "/share");
+	for (p = RSHARE; *p; p++) if (*p == '\\') *p = '/';
 	putenv(RSHARE);
 
 	snprintf(Rversion, 25, "R_VERSION=%s.%s", R_MAJOR, R_MINOR);
@@ -274,7 +277,7 @@ int rcmdfn (int cmdarg, int argc, char **argv)
 	if ( (p = getenv("PERL5LIB")) ) strcat(PERL5LIB, p);
 	putenv(PERL5LIB);
 
-	strcpy(TEXINPUTS, "TEXINPUTS=");
+	strcpy(TEXINPUTS, "TEXINPUTS=.;");
 	strcat(TEXINPUTS, RHome); strcat(TEXINPUTS, "\\share\\texmf;");
 	if ( (p = getenv("TEXINPUTS")) ) strcat(TEXINPUTS, p);
 	putenv(TEXINPUTS);
