@@ -55,8 +55,16 @@ row.names.default <- function(x) if(!is.null(dim(x))) rownames(x)# else NULL
     }
     else if (length(value) != n)
 	stop("invalid 'row.names' length")
-    if (any(duplicated(value)))
+    if (any(duplicated(value))) {
+        nonuniq <- sort(unique(value[duplicated(value)]))
+        warning(ngettext(length(nonuniq),
+                         sprintf("non-unique value when setting 'row.names': %s",
+                                 sQuote(nonuniq[1])),
+                         sprintf("non-unique values when setting 'row.names': %s",
+                                 paste(sQuote(nonuniq), collapse = ", "))),
+                domain = NA, call. = FALSE)
 	stop("duplicate 'row.names' are not allowed")
+    }
     if (any(is.na(value)))
 	stop("missing values in 'row.names' are not allowed")
     attr(x, "row.names") <- value
@@ -399,7 +407,7 @@ data.frame <-
                 vnames[[i]] <- tmpname
             }
         } # end of ncols[i] <= 1
-	if(is.null(row.names) && nrows[i] > 0L) {
+	if(missing(row.names) && nrows[i] > 0L) {
             rowsi <- attr(xi, "row.names")
             ## old way to mark optional names
             if(!(rowsi[[1L]] %in% ""))
@@ -500,9 +508,9 @@ data.frame <-
         ## not quite the same as the 1/2-arg case, as 'drop' is used.
         if(missing(j) && drop && length(x) == 1L) return(.subset2(x, 1L))
         y <- if(missing(j)) x else .subset(x, j)
-        if(drop && length(y) == 1L) return(.subset2(y, 1L))
 	cols <- names(y)
 	if(any(is.na(cols))) stop("undefined columns selected")
+        if(drop && length(y) == 1L) return(.subset2(y, 1L))
         if(any(duplicated(cols))) names(y) <- make.unique(cols)
         nrow <- .row_names_info(x, 2L)
         if(drop && !mdrop && nrow == 1L)
@@ -573,6 +581,7 @@ data.frame <-
         rows <- rows[i]
 	if((ina <- any(is.na(rows))) | (dup <- any(duplicated(rows)))) {
 	    ## both will coerce integer 'rows' to character:
+	    if (!dup && is.character(rows)) dup <- "NA" %in% rows
 	    if(ina)
 		rows[is.na(rows)] <- "NA"
 	    if(dup)
@@ -587,7 +596,7 @@ data.frame <-
     x
 }
 
-"[[.data.frame" <- function(x, ..., exact=NA)
+"[[.data.frame" <- function(x, ..., exact=TRUE)
 {
     ## use in-line functions to refer to the 1st and 2nd ... arguments
     ## explicitly. Also will check for wrong number or empty args
@@ -707,6 +716,7 @@ data.frame <-
         if(any(is.na(j)))
             stop("missing values are not allowed in subscripted assignments of data frames")
 	if(is.character(j)) {
+            if("" %in% j) stop("column name \"\" cannot match any column")
 	    jj <- match(j, names(x))
 	    nnew <- sum(is.na(jj))
 	    if(nnew > 0L) {
@@ -801,14 +811,18 @@ data.frame <-
     ncolv <- dimv[2L]
     jvseq <- seq_len(p)
     if(ncolv < p) jvseq <- rep(seq_len(ncolv), length.out = p)
-    else if(ncolv > p)
+    else if(ncolv > p) {
 	warning(gettextf("provided %d variables to replace %d variables",
                          ncolv, p), domain = NA)
+        new.cols <- new.cols[seq_len(p)]
+    }
     if(length(new.cols)) {
         ## extend and name now, as assignment of NULL may delete cols later.
         nm <- names(x)
         rows <- .row_names_info(x, 0L)
+        a <- attributes(x); a["names"] <- NULL
         x <- c(x, vector("list", length(new.cols)))
+        attributes(x) <- a
         names(x) <- c(nm, new.cols)
         attr(x, "row.names") <- rows
     }
@@ -917,10 +931,10 @@ data.frame <-
 	stop("non-existent rows not allowed")
 
     if(is.character(j)) {
+        if("" %in% j) stop("column name \"\" cannot match any column")
 	jseq <- match(j, names(x))
 	if(any(is.na(jseq)))
-	    stop("replacing element in non-existent column: ",
-                 j[is.na(jseq)])
+	    stop("replacing element in non-existent column: ", j[is.na(jseq)])
     }
     else if(is.logical(j) || min(j) < 0L)
 	jseq <- seq_along(x)[j]
@@ -1010,7 +1024,7 @@ rbind.data.frame <- function(..., deparse.level = 1)
     match.names <- function(clabs, nmi)
     {
 	if(identical(clabs, nmi)) NULL
-	else if(length(nmi) == length(clabs) && all(match(nmi, clabs, 0L))) {
+	else if(length(nmi) == length(clabs) && all(nmi %in% clabs)) {
             ## we need 1-1 matches here
 	    m <- pmatch(nmi, clabs, 0L)
             if(any(m == 0L))
@@ -1202,17 +1216,22 @@ rbind.data.frame <- function(..., deparse.level = 1)
 ### coercion and print methods
 
 print.data.frame <-
-    function(x, ..., digits = NULL, quote = FALSE, right = TRUE)
+    function(x, ..., digits = NULL, quote = FALSE, right = TRUE,
+	     row.names = TRUE)
 {
+    n <- length(row.names(x))
     if(length(x) == 0L) {
-	cat("NULL data frame with", length(row.names(x)), "rows\n")
-    } else if(length(row.names(x)) == 0L) {
+	cat("NULL data frame with", n, "rows\n")
+    } else if(n == 0L) {
 	print.default(names(x), quote = FALSE)
 	cat("<0 rows> (or 0-length row.names)\n")
     } else {
-	## avoiding picking up e.g. format.AsIs
-	print(as.matrix(format.data.frame(x, digits=digits, na.encode=FALSE)),
-              ..., quote = quote, right = right)
+	## format.<*>() : avoiding picking up e.g. format.AsIs
+	m <- as.matrix(format.data.frame(x, digits=digits, na.encode=FALSE))
+	if(!isTRUE(row.names))
+	    dimnames(m)[[1]] <- if(identical(row.names,FALSE))
+		rep.int("", n) else row.names
+	print(m, ..., quote = quote, right = right)
     }
     invisible(x)
 }
