@@ -45,15 +45,21 @@ as.Date.character <- function(x, format="", ...)
             if(is.na(xx)) f <- "%Y-%m-%d" # all NAs
         }
 	if(is.na(xx) ||
-	   !is.na(strptime(xx, f <- "%Y-%m-%d")) ||
-	   !is.na(strptime(xx, f <- "%Y/%m/%d"))
+	   !is.na(strptime(xx, f <- "%Y-%m-%d", tz="GMT")) ||
+	   !is.na(strptime(xx, f <- "%Y/%m/%d", tz="GMT"))
            ) return(strptime(x, f))
 	stop("character string is not in a standard unambiguous format")
     }
-    res <- if(missing(format)) fromchar(x) else strptime(x, format)
+    res <- if(missing(format)) fromchar(x) else strptime(x, format, tz="GMT")
     .Internal(POSIXlt2Date(res))
 }
 
+as.Date.numeric <- function(x, origin, ...)
+{
+    if(missing(origin)) stop("'origin' must be supplied")
+    as.Date(origin, ...) + x
+}
+
 as.Date.default <- function(x, ...)
 {
     if(inherits(x, "Date")) return(x)
@@ -303,7 +309,7 @@ cut.Date <-
         if(length(by2) > 2 || length(by2) < 1)
             stop("invalid specification of 'breaks'")
 	valid <-
-	    pmatch(by2[length(by2)], c("days", "weeks", "months", "years"))
+	    pmatch(by2[length(by2)], c("days", "weeks", "months", "years", "quarters"))
 	if(is.na(valid)) stop("invalid specification of 'breaks'")
 	start <- as.POSIXlt(min(x, na.rm=TRUE))
 	if(valid == 1) incr <- 1
@@ -313,13 +319,39 @@ cut.Date <-
 		start$mday <- start$mday + ifelse(start$wday > 0, 1, -6)
 	    incr <- 7
 	}
-	if(valid == 3) { start$mday <- 1; incr <- 31 }
-	if(valid == 4) { start$mon <- 0; start$mday <- 1; incr <- 366 }
+    if(valid == 3) {
+        start$mday <- 1
+        end <- as.POSIXlt(max(x, na.rm = TRUE))
+        step <- ifelse(length(by2) == 2, as.integer(by2[1]), 1)
+        end <- as.POSIXlt(end + (31 * step * 86400))
+        end$mday <- 1
+        breaks <- as.Date(seq(start, end, breaks))
+    } else if(valid == 4) {
+        start$mon <- 0
+        start$mday <- 1
+        end <- as.POSIXlt(max(x, na.rm = TRUE))
+        step <- ifelse(length(by2) == 2, as.integer(by2[1]), 1)
+        end <- as.POSIXlt(end + (366 * step * 86400))
+        end$mon <- 0
+        end$mday <- 1
+        breaks <- as.Date(seq(start, end, breaks))
+    } else if(valid == 5) {
+        qtr <- rep(c(0, 3, 6, 9), each = 3)
+        start$mon <- qtr[start$mon + 1]
+        start$mday <- 1
+        end <- as.POSIXlt(max(x, na.rm = TRUE))
+        step <- ifelse(length(by2) == 2, as.integer(by2[1]), 1)
+        end <- as.POSIXlt(end + (93 * step * 86400))
+        end$mon <- qtr[end$mon + 1]
+        end$mday <- 1
+        breaks <- as.Date(seq(start, end, paste(step * 3, "months")))
+    } else {
         start <- .Internal(POSIXlt2Date(start))
         if (length(by2) == 2) incr <- incr * as.integer(by2[1])
-	maxx <- max(x, na.rm = TRUE)
-	breaks <- seq.int(start, maxx + incr, breaks)
-	breaks <- breaks[1:(1+max(which(breaks < maxx)))]
+	    maxx <- max(x, na.rm = TRUE)
+        breaks <- seq.int(start, maxx + incr, breaks)
+        breaks <- breaks[1:(1+max(which(breaks < maxx)))]
+      }
     } else stop("invalid specification of 'breaks'")
     res <- cut(unclass(x), unclass(breaks), labels = labels,
                right = right, ...)
