@@ -119,7 +119,7 @@ png <- function(filename = "Rplot%03d.png", width = 480, height = 480,
                 units = "px",
                 pointsize = 12, bg = "white", res = NA, restoreConsole = TRUE)
 {
-    checkIntFormat(filename)
+    if(!checkIntFormat(filename)) stop("invalid 'filename'")
     units <- match.arg(units, c("in", "px", "cm", "mm"))
     if(units != "px" && is.na(res))
         stop("'res' must be specified unless 'units = \"px\"'")
@@ -206,7 +206,7 @@ bringToTop <- function(which = dev.cur(), stay = FALSE)
     invisible(.Internal(bringToTop(as.integer(which), as.logical(stay))))
 }
 
-savePlot <- function(filename = paste("Rplot", type, sep="."),
+savePlot <- function(filename = "Rplot",
                      type = c("wmf", "emf", "png", "jpg", "jpeg", "bmp",
                      "tif", "tiff", "ps", "eps", "pdf"),
                      device = dev.cur(),
@@ -219,6 +219,7 @@ savePlot <- function(filename = paste("Rplot", type, sep="."),
     devname <- names(devlist)[devcur]
     if(devname != "windows") stop("can only copy from 'windows' devices")
     if(filename == "clipboard" && type == "wmf") filename <- ""
+    else if(regexpr("\\.",filename) < 0) filename <- paste(filename,type,sep=".")
     invisible(.External(CsavePlot, device, filename, type, restoreConsole))
 }
 
