@@ -31,8 +31,8 @@ curve <- function(expr, from=NULL, to=NULL, n=101, add=FALSE, type="l",
     if (is.null(xlim))
 	delayedAssign("lims",
 		  {pu <- par("usr")[1:2]
-		   ll <- if(par("xlog")) 10^pu else pu
-		   if(par("xaxs") == "r") extendrange(ll, f = -1/27) else ll })
+		   if(par("xaxs") == "r") pu <- extendrange(pu, f = -1/27)
+		   if(par("xlog")) 10^pu else pu })
     else lims <- xlim
     if(is.null(from)) from <- lims[1]
     if(is.null(to))     to <- lims[2]
