@@ -1,3 +1,19 @@
+#  File src/library/graphics/R/hist.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 hist <- function(x, ...) UseMethod("hist")
 
 hist.default <-
@@ -42,7 +58,7 @@ hist.default <-
 	} else if(is.function(breaks)) {
 	    breaks <- breaks(x)
 	}
-	if(!is.numeric(breaks) || is.na(breaks) || breaks < 2)
+	if(!is.numeric(breaks) || !is.finite(breaks) || breaks < 1)
 	    stop("invalid number of 'breaks'")
 	breaks <- pretty (range(x), n = breaks, min.n = 1)
 	nB <- length(breaks)
@@ -56,8 +72,9 @@ hist.default <-
     equidist <- !use.br || diff(range(h)) < 1e-7 * mean(h)
     if (!use.br && any(h <= 0))
 	stop("'breaks' are not strictly increasing")
+    freq1 <- freq # we want to do missing(freq) later
     if (is.null(freq)) {
-	freq <- if(!missing(probability)) !as.logical(probability) else equidist
+	freq1 <- if(!missing(probability)) !as.logical(probability) else equidist
     } else if(!missing(probability) && any(probability == freq))
 	stop("'probability' is an alias for '!freq', however they differ.")
 
@@ -80,7 +97,7 @@ hist.default <-
     storage.mode(x) <- "double"
     storage.mode(fuzzybreaks) <- "double"
     ## With the fuzz adjustment above, the "right" and "include"
-    ## arguments are really irrelevant
+    ## arguments are often irrelevant (not with integer data!)
     counts <- .C("bincount",
 		 x,
 		 as.integer(n),
@@ -102,7 +119,7 @@ hist.default <-
 			xname = xname, equidist = equidist),
 		   class="histogram")
     if (plot) {
-	plot(r, freq = freq, col = col, border = border,
+	plot(r, freq = freq1, col = col, border = border,
 	     angle = angle, density = density,
 	     main = main, xlim = xlim, ylim = ylim, xlab = xlab, ylab = ylab,
 	     axes = axes, labels = labels, ...)
@@ -111,7 +128,8 @@ hist.default <-
     else { ## plot is FALSE
         ## make an effort to warn about "non sensical" arguments:
         nf <- names(formals()) ## all formals but those:
-        nf <- nf[is.na(match(nf, c("x", "breaks", "freq", "nclass", "plot", "probability")))]
+	nf <- nf[is.na(match(nf, c("x", "breaks", "nclass", "plot",
+				   "include.lowest", "right")))]
         missE <- lapply(nf, function(n)
                         substitute(missing(.), list(. = as.name(n))))
         not.miss <- ! sapply(missE, eval, envir = environment())
