@@ -1,3 +1,19 @@
+#  File src/library/stats/R/ar.burg.mts.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 "ar.burg.mts" <-
 function (x, aic = TRUE, order.max = NULL, na.action = na.fail,
     demean = TRUE, series = NULL, var.method = 1, ...)
@@ -17,7 +33,7 @@ function (x, aic = TRUE, order.max = NULL, na.action = na.fail,
     n.used <- nrow(x)
     if (demean) {
         x.mean <- colMeans(x)
-        x <- sweep(x, 2, x.mean)
+        x <- sweep(x, 2, x.mean, check.margin=FALSE)
     }
     else x.mean <- rep(0, nser)
     order.max <- if (is.null(order.max))
