@@ -1,4 +1,22 @@
-#### copyright (C) 1994-2001 W. N. Venables and B. D. Ripley
+#  File src/library/stats/R/bandwidths.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
+### copyright (C) 1994-2001 W. N. Venables and B. D. Ripley
+### This version distributed under GPL (version 2 or later)
+
 
 #====           bandwidth selection rules              ====
 
@@ -30,31 +48,28 @@ bw.SJ <- function(x, nb = 1000, lower = 0.1*hmax, upper = hmax,
     fSD <- function(h, x, alph2, c1, n, d)
         (c1/SDh(x, alph2 * h^(5/7), n, d))^(1/5) - h
     SDh <- function(x, h, n, d)
-        .C("band_phi4_bin",
+        .C(R_band_phi4_bin,
            as.integer(n),
            as.integer(length(x)),
            as.double(d),
            x,
            as.double(h),
-           u = double(1),
-           PACKAGE="stats")$u
+           u = double(1))$u
     TDh <- function(x, h, n, d)
-        .C("band_phi6_bin",
+        .C(R_band_phi6_bin,
            as.integer(n),
            as.integer(length(x)),
            as.double(d),
            x,
            as.double(h),
-           u = double(1),
-           PACKAGE="stats")$u
+           u = double(1))$u
 
-    Z <- .C("band_den_bin",
+    Z <- .C(R_band_den_bin,
             as.integer(n),
             as.integer(nb),
             d = double(1),
             x,
-            cnt = integer(nb),
-            PACKAGE="stats")
+            cnt = integer(nb))
     d <- Z$d; cnt <- as.integer(Z$cnt)
     hmax <- 1.144 * sqrt(var(x)) * n^(-1/5)
     scale <- min(sqrt(var(x)), IQR(x)/1.349)
@@ -70,9 +85,19 @@ bw.SJ <- function(x, nb = 1000, lower = 0.1*hmax, upper = hmax,
         alph2 <- 1.357*(SDh(cnt, a, n, d)/TD)^(1/7)
         if(!is.finite(alph2))
             stop("sample is too sparse to find alph2")
-        if (fSD(lower, cnt, alph2, c1, n, d) *
-            fSD(upper, cnt, alph2, c1, n, d) > 0)
-            stop("no solution in the specified range of bandwidths")
+        itry <- 1
+	while (fSD(lower, cnt, alph2, c1, n, d) *
+	       fSD(upper, cnt, alph2, c1, n, d) > 0) {
+	    if(itry > 20 || !(missing(lower) && missing(upper)))
+		stop("no solution in the specified range of bandwidths")
+	    if(itry %% 2)
+		upper <- upper * 1.2
+	    else lower <- lower / 1.2
+	    if(getOption("verbose"))
+		message(gettextf("increasing bw.SJ() search interval (%d)",
+                        itry), sep='', domain = NA)
+	    itry <- itry + 1
+	}
         res <- uniroot(fSD, c(lower, upper), tol=0.1*lower,
                        x=cnt, alph2=alph2, c1=c1, n=n, d=d)$root
     }
@@ -86,24 +111,22 @@ bw.ucv <- function(x, nb = 1000, lower = 0.1*hmax, upper = hmax)
     if(!is.numeric(x)) stop("invalid 'x'")
 
     fucv <- function(h, x, n, d)
-        .C("band_ucv_bin",
+        .C(R_band_ucv_bin,
            as.integer(n),
            as.integer(length(x)),
            as.double(d),
            x,
            as.double(h),
-           u = double(1),
-           PACKAGE="stats")$u
+           u = double(1))$u
 
     hmax <- 1.144 * sqrt(var(x)) * n^(-1/5)
     storage.mode(x) <- "double"
-    Z <- .C("band_den_bin",
+    Z <- .C(R_band_den_bin,
             as.integer(n),
             as.integer(nb),
             d = double(1),
             x,
-            cnt = integer(nb),
-            PACKAGE="stats")
+            cnt = integer(nb))
     d <- Z$d; cnt <- as.integer(Z$cnt)
     h <- optimize(fucv, c(lower, upper), tol=0.1*lower,
                   x=cnt, n=n, d=d)$minimum
@@ -118,25 +141,22 @@ bw.bcv <- function(x, nb = 1000, lower = 0.1*hmax, upper = hmax)
     if(!is.numeric(x)) stop("invalid 'x'")
 
     fbcv <- function(h, x, n, d)
-        .C("band_bcv_bin",
+        .C(R_band_bcv_bin,
            as.integer(n),
            as.integer(length(x)),
            as.double(d),
            x,
            as.double(h),
-           u = double(1),
-           PACKAGE="stats")$u
+           u = double(1))$u
 
     hmax <- 1.144 * sqrt(var(x)) * n^(-1/5)
     storage.mode(x) <- "double"
-    Z <- .C("band_den_bin",
+    Z <- .C(R_band_den_bin,
             as.integer(n),
             as.integer(nb),
             d = double(1),
             x,
-            cnt = integer(nb),
-            PACKAGE="stats"
-            )
+            cnt = integer(nb))
     d <- Z$d; cnt <- as.integer(Z$cnt)
     h <- optimize(fbcv, c(lower, upper), tol=0.1*lower,
                   x=cnt, n=n, d=d)$minimum
