@@ -1,9 +1,26 @@
+#  File src/library/stats/R/family.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 family <- function(object, ...) UseMethod("family")
 
 print.family <- function(x, ...)
 {
     cat("\nFamily:", x$family, "\n")
     cat("Link function:", x$link, "\n\n")
+    invisible(x)
 }
 
 power <- function(lambda = 1)
