@@ -109,9 +109,9 @@ profiler.nls <- function(fitted, ...)
 		 } else {
 		     iv <- nls_port_fit(fittedModel, startPars[vary],
 					lower[vary], upper[vary], ctrl, trace)
-		     dev <- if(!iv[1] %in% 3:6) 
-			NA_real_ 
-		     else 
+		     dev <- if(!iv[1] %in% 3:6)
+			NA_real_
+		     else
 			fittedModel$deviance()
 		 }
 		 profiledModel <- fittedModel
@@ -225,18 +225,12 @@ plot.profile.nls <- function(x, levels, conf = c(99, 95, 90, 80, 50)/100,
 {
     obj <- x
     dfres <- attr(obj, "summary")$df[2]
-    confstr <- NULL
-    if(missing(levels)) {
+    if(missing(levels))
         levels <- sqrt(qf(pmax(0, pmin(1, conf)), 1, dfres))
-        confstr <- paste(format(100 * conf), "%", sep = "")
-    }
     if(any(levels <= 0)) {
         levels <- levels[levels > 0]
         warning("levels truncated to positive values only")
     }
-    if(is.null(confstr)) {
-        confstr <- paste(format(100 * pf(levels^2, 1, dfres)), "%", sep = "")
-    }
     mlev <- max(levels) * 1.05
     nm <- names(obj)
     opar <- par(mar = c(5, 4, 1, 1) + 0.1)
