@@ -44,15 +44,16 @@ static R_StringBuffer cbuff = {NULL, 0, MAXELTSIZE};
  * then it is alloc-ed and the second pass stuffs the information in.
  */
 
-/* Note that NA_STRING is not handled separately here.  This is 
+/* Note that NA_STRING is not handled separately here.  This is
    deliberate -- see ?paste -- and implicitly coerces it to "NA"
 */
 SEXP attribute_hidden do_paste(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP ans, collapse, sep, x;
-    int i, j, k, maxlen, nx, pwidth, sepw, ienc;
-    const char *s, *csep, *cbuf; char *buf;
-    Rboolean allKnown, anyKnown, sepASCII, sepKnown;
+    int i, j, k, maxlen, nx, pwidth, sepw, u_sepw, ienc;
+    const char *s, *csep, *cbuf, *u_csep=NULL;
+    char *buf;
+    Rboolean allKnown, anyKnown, sepASCII, sepKnown, use_UTF8, sepUTF8;
 
     checkArity(op, args);
 
@@ -67,16 +68,17 @@ SEXP attribute_hidden do_paste(SEXP call, SEXP op, SEXP args, SEXP env)
     if (!isVectorList(x))
 	error(_("invalid first argument"));
     nx = length(x);
-	
+
 
     sep = CADR(args);
     if (!isString(sep) || LENGTH(sep) <= 0 || STRING_ELT(sep, 0) == NA_STRING)
 	error(_("invalid separator"));
     sep = STRING_ELT(sep, 0);
     csep = translateChar(sep);
-    sepw = strlen(csep); /* not LENGTH as might contain \0 */
+    u_sepw = sepw = strlen(csep);
     sepASCII = strIsASCII(csep);
     sepKnown = ENC_KNOWN(sep) > 0;
+    sepUTF8 = IS_UTF8(sep);
 
     collapse = CADDR(args);
     if (!isNull(collapse))
@@ -85,7 +87,7 @@ SEXP attribute_hidden do_paste(SEXP call, SEXP op, SEXP args, SEXP env)
 	    error(_("invalid '%s' argument"), "collapse");
     if(nx == 0)
 	return (!isNull(collapse)) ? mkString("") : allocVector(STRSXP, 0);
-    
+
 
     /* Maximum argument length, coerce if needed */
 
@@ -121,38 +123,65 @@ SEXP attribute_hidden do_paste(SEXP call, SEXP op, SEXP args, SEXP env)
 	 * declared encoding, we should mark.
 	 * Need to be careful only to include separator if it is used.
 	 */
-	anyKnown = FALSE; allKnown = TRUE;
+	anyKnown = FALSE; allKnown = TRUE; use_UTF8 = FALSE;
 	if(nx > 1) {
-	    allKnown = allKnown && (sepKnown || sepASCII);
-	    anyKnown = anyKnown || sepKnown;
+	    allKnown = sepKnown || sepASCII;
+	    anyKnown = sepKnown;
+	    use_UTF8 = sepUTF8;
 	}
-	
+
 	pwidth = 0;
 	for (j = 0; j < nx; j++) {
 	    k = length(VECTOR_ELT(x, j));
-	    if (k > 0)
-		pwidth += strlen(translateChar(STRING_ELT(VECTOR_ELT(x, j),
-							  i % k)));
+	    if (k > 0) {
+		SEXP cs = STRING_ELT(VECTOR_ELT(x, j), i % k);
+		if(IS_UTF8(cs)) use_UTF8 = TRUE;
+	    }
 	}
-	pwidth += (nx - 1) * sepw;
+	for (j = 0; j < nx; j++) {
+	    k = length(VECTOR_ELT(x, j));
+	    if (k > 0) {
+		if(use_UTF8)
+		    pwidth += strlen(translateCharUTF8(STRING_ELT(VECTOR_ELT(x, j), i % k)));
+		else
+		    pwidth += strlen(translateChar(STRING_ELT(VECTOR_ELT(x, j), i % k)));
+	    }
+	}
+	if (use_UTF8 && !u_csep) {
+	    u_csep = translateCharUTF8(sep);
+	    u_sepw = strlen(u_csep);
+	}
+	pwidth += (nx - 1) * (use_UTF8 ? u_sepw : sepw);
 	cbuf = buf = R_AllocStringBuffer(pwidth, &cbuff);
 	for (j = 0; j < nx; j++) {
 	    k = length(VECTOR_ELT(x, j));
 	    if (k > 0) {
 		SEXP cs = STRING_ELT(VECTOR_ELT(x, j), i % k);
-		s = translateChar(cs);
-                strcpy(buf, s);
-		buf += strlen(s);
-		allKnown = allKnown && (strIsASCII(s) || (ENC_KNOWN(cs)> 0));
-		anyKnown = anyKnown || (ENC_KNOWN(cs)> 0);
+		if (use_UTF8) {
+		    s = translateCharUTF8(cs);
+		    strcpy(buf, s);
+		    buf += strlen(s);
+		} else {
+		    s = translateChar(cs);
+		    strcpy(buf, s);
+		    buf += strlen(s);
+		    allKnown = allKnown && (strIsASCII(s) || (ENC_KNOWN(cs)> 0));
+		    anyKnown = anyKnown || (ENC_KNOWN(cs)> 0);
+		}
 	    }
 	    if (j != nx - 1 && sepw != 0) {
-	        strcpy(buf, csep);
-		buf += sepw;
+		if (use_UTF8) {
+		    strcpy(buf, u_csep);
+		    buf += u_sepw;
+		} else {
+		    strcpy(buf, csep);
+		    buf += sepw;
+		}
 	    }
 	}
 	ienc = 0;
-	if(anyKnown && allKnown) {
+	if(use_UTF8) ienc = CE_UTF8;
+	else if(anyKnown && allKnown) {
 	    if(known_to_be_latin1) ienc = CE_LATIN1;
 	    if(known_to_be_utf8) ienc = CE_UTF8;
 	}
@@ -161,34 +190,46 @@ SEXP attribute_hidden do_paste(SEXP call, SEXP op, SEXP args, SEXP env)
 
     /* Now collapse, if required. */
 
-    if(collapse != R_NilValue && (nx = LENGTH(ans)) > 0) {
+    if(collapse != R_NilValue && (nx = LENGTH(ans)) > 0) {	
 	sep = STRING_ELT(collapse, 0);
-	csep = translateChar(sep);
+	use_UTF8 = IS_UTF8(sep);
+	for (i = 0; i < nx; i++) 
+	    if(IS_UTF8(STRING_ELT(ans, i))) use_UTF8 = TRUE;
+	if(use_UTF8)
+	    csep = translateCharUTF8(sep);
+	else
+	    csep = translateChar(sep);
 	sepw = strlen(csep);
 	anyKnown = ENC_KNOWN(sep) > 0;
 	allKnown = anyKnown || strIsASCII(csep);
 	pwidth = 0;
-	/* 'ans' is already translated */
 	for (i = 0; i < nx; i++)
-	    pwidth += strlen(CHAR(STRING_ELT(ans, i)));
+	    if(use_UTF8)
+		pwidth += strlen(translateCharUTF8(STRING_ELT(ans, i)));
+	    else
+		pwidth += strlen(CHAR(STRING_ELT(ans, i)));
 	pwidth += (nx - 1) * sepw;
 	cbuf = buf = R_AllocStringBuffer(pwidth, &cbuff);
 	for (i = 0; i < nx; i++) {
 	    if(i > 0) {
-	        strcpy(buf, csep);
+		strcpy(buf, csep);
 		buf += sepw;
 	    }
-	    s = CHAR(STRING_ELT(ans, i));
-            strcpy(buf, s);
+	    if(use_UTF8)
+		s = translateCharUTF8(STRING_ELT(ans, i));
+	    else /* already translated */
+		s = CHAR(STRING_ELT(ans, i));
+	    strcpy(buf, s);
 	    while (*buf)
 		buf++;
-	    allKnown = allKnown && 
+	    allKnown = allKnown &&
 		(strIsASCII(s) || (ENC_KNOWN(STRING_ELT(ans, i))> 0));
 	    anyKnown = anyKnown || (ENC_KNOWN(STRING_ELT(ans, i))> 0);
 	}
-        UNPROTECT(1);
-	ienc = 0;
-	if(anyKnown && allKnown) {
+	UNPROTECT(1);
+	ienc = CE_NATIVE;
+	if(use_UTF8) ienc = CE_UTF8;
+	else if(anyKnown && allKnown) {
 	    if(known_to_be_latin1) ienc = CE_LATIN1;
 	    if(known_to_be_utf8) ienc = CE_UTF8;
 	}
@@ -215,7 +256,7 @@ SEXP attribute_hidden do_filepath(SEXP call, SEXP op, SEXP args, SEXP env)
 	error(_("invalid first argument"));
     nx = length(x);
     if(nx == 0) return allocVector(STRSXP, 0);
-	
+
 
     sep = CADR(args);
     if (!isString(sep) || LENGTH(sep) <= 0 || STRING_ELT(sep, 0) == NA_STRING)
@@ -254,19 +295,19 @@ SEXP attribute_hidden do_filepath(SEXP call, SEXP op, SEXP args, SEXP env)
 	pwidth = 0;
 	for (j = 0; j < nx; j++) {
 	    k = length(VECTOR_ELT(x, j));
-	    pwidth += strlen(CHAR(STRING_ELT(VECTOR_ELT(x, j), i % k)));
+	    pwidth += strlen(translateChar(STRING_ELT(VECTOR_ELT(x, j), i % k)));
 	}
 	pwidth += (nx - 1) * sepw;
 	cbuf = buf = R_AllocStringBuffer(pwidth, &cbuff);
 	for (j = 0; j < nx; j++) {
 	    k = length(VECTOR_ELT(x, j));
 	    if (k > 0) {
-		s = CHAR(STRING_ELT(VECTOR_ELT(x, j), i % k));
-                strcpy(buf, s);
+		s = translateChar(STRING_ELT(VECTOR_ELT(x, j), i % k));
+		strcpy(buf, s);
 		buf += strlen(s);
 	    }
 	    if (j != nx - 1 && sepw != 0) {
-	        strcpy(buf, csep);
+		strcpy(buf, csep);
 		buf += sepw;
 	    }
 	}
@@ -277,8 +318,8 @@ SEXP attribute_hidden do_filepath(SEXP call, SEXP op, SEXP args, SEXP env)
     return ans;
 }
 
-/* format.default(x, trim, digits, nsmall, width, justify, na.encode, 
-                  scientific) */
+/* format.default(x, trim, digits, nsmall, width, justify, na.encode,
+		  scientific) */
 SEXP attribute_hidden do_format(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP l, x, y, swd;
@@ -289,7 +330,7 @@ SEXP attribute_hidden do_format(SEXP call, SEXP op, SEXP args, SEXP env)
 
     checkArity(op, args);
     PrintDefaults(env);
-    
+
     if (!isVector(x = CAR(args)))
 	error(_("first argument must be atomic"));
     args = CDR(args);
@@ -301,7 +342,7 @@ SEXP attribute_hidden do_format(SEXP call, SEXP op, SEXP args, SEXP env)
 
     if (!isNull(CAR(args))) {
 	digits = asInteger(CAR(args));
-	if (digits == NA_INTEGER || digits < R_MIN_DIGITS_OPT 
+	if (digits == NA_INTEGER || digits < R_MIN_DIGITS_OPT
 	    || digits > R_MAX_DIGITS_OPT)
 	    error(_("invalid '%s' argument"), "digits");
 	R_print.digits = digits;
@@ -394,7 +435,7 @@ SEXP attribute_hidden do_format(SEXP call, SEXP op, SEXP args, SEXP env)
 	    const char *s; char *buff, *q;
 	    int b, b0, cnt = 0, j;
 	    SEXP s0, xx;
-	    
+
 	    /* This is clumsy, but it saves rewriting and re-testing
 	       this complex code */
 	    PROTECT(xx = duplicate(x));
@@ -403,8 +444,8 @@ SEXP attribute_hidden do_format(SEXP call, SEXP op, SEXP args, SEXP env)
 		s = translateChar(tmp);
 		if(s != CHAR(tmp)) SET_STRING_ELT(xx, i, mkChar(s));
 	    }
-		
-	    w = wd; 
+
+	    w = wd;
 	    if (adj != Rprt_adj_none) {
 		for (i = 0; i < n; i++)
 		    if (STRING_ELT(xx, i) != NA_STRING)
@@ -482,7 +523,7 @@ SEXP attribute_hidden do_formatinfo(SEXP call, SEXP op, SEXP args, SEXP env)
     digits = asInteger(CADR(args));
     if (!isNull(CADR(args))) {
 	digits = asInteger(CADR(args));
-	if (digits == NA_INTEGER || digits < R_MIN_DIGITS_OPT 
+	if (digits == NA_INTEGER || digits < R_MIN_DIGITS_OPT
 	    || digits > R_MAX_DIGITS_OPT)
 	    error(_("invalid '%s' argument"), "digits");
 	R_print.digits = digits;
