@@ -1056,7 +1056,7 @@ static BBOX RenderSymbolStr(const char *str, int draw, mathContext *mc,
 		    font = prevfont;
 		    SetFont(prevfont, gc);
 		}
-		glyphBBox = GlyphBBox((int)wc, gc, dd);
+		glyphBBox = GlyphBBox((unsigned int) wc, gc, dd);
 		if (UsingItalics(gc))
 		    bboxItalic(glyphBBox) =
 			ItalicFactor * bboxHeight(glyphBBox);
@@ -1090,7 +1090,7 @@ static BBOX RenderSymbolStr(const char *str, int draw, mathContext *mc,
 		    font = prevfont;
 		    SetFont(prevfont, gc);
 		}
-		glyphBBox = GlyphBBox(*s, gc, dd);
+		glyphBBox = GlyphBBox((unsigned char) *s, gc, dd);
 		if (UsingItalics(gc))
 		    bboxItalic(glyphBBox) =
 			ItalicFactor * bboxHeight(glyphBBox);
@@ -1164,7 +1164,8 @@ static BBOX RenderStr(const char *str, int draw, mathContext *mc,
 	    mbstate_t mb_st;
 	    mbs_init(&mb_st);
 	    while ((used = Mbrtowc(&wc, p, n, &mb_st)) > 0) {
-		glyphBBox = GlyphBBox(wc, gc, dd);
+		/* On Windows could have sign extension here */
+		glyphBBox = GlyphBBox((unsigned int) wc, gc, dd);
 		resultBBox = CombineBBoxes(resultBBox, glyphBBox);
 		p += used; n -= used; nc++;
 	    }
@@ -1173,7 +1174,8 @@ static BBOX RenderStr(const char *str, int draw, mathContext *mc,
 	{
 	    const char *s = str;
 	    while (*s) {
-		glyphBBox = GlyphBBox(*s, gc, dd);
+		/* Watch for sign extension here - fixed > 2.7.1 */
+		glyphBBox = GlyphBBox((unsigned char) *s, gc, dd);
 		resultBBox = CombineBBoxes(resultBBox, glyphBBox);
 		s++; nc++;
 	    }
