@@ -74,7 +74,7 @@ SEXP attribute_hidden do_relop_dflt(SEXP call, SEXP op, SEXP x, SEXP y)
 	return ans;
     }
 
-    /* That symbols and calls were allowwed was undocumented prior to
+    /* That symbols and calls were allowed was undocumented prior to
        R 2.5.0.  We deparse them as deparse() would, minus attributes */
     if ((iS = isSymbol(x)) || TYPEOF(x) == LANGSXP) {
 	SEXP tmp = allocVector(STRSXP, 1);
@@ -448,7 +448,7 @@ static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
     n = (n1 > n2) ? n1 : n2;
     PROTECT(s1);
     PROTECT(s2);
-    ans = allocVector(LGLSXP, n);
+    PROTECT(ans = allocVector(LGLSXP, n));
 
     switch (code) {
     case EQOP:
@@ -457,13 +457,8 @@ static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 	    c2 = STRING_ELT(s2, i % n2);
 	    if (c1 == NA_STRING || c2 == NA_STRING)
 		LOGICAL(ans)[i] = NA_LOGICAL;
- 	    else if (c1 == c2)  /* This will pretest all cached strings */
-		LOGICAL(ans)[i] = 1;
 	    else
-		if (Seql(c1, c2))
-		    LOGICAL(ans)[i] = 1;
-		else
-		    LOGICAL(ans)[i] = 0;
+		LOGICAL(ans)[i] = Seql(c1, c2) ? 1 : 0;
 	}
 	break;
     case NEOP:
@@ -472,13 +467,8 @@ static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 	    c2 = STRING_ELT(s2, i % n2);
 	    if (c1 == NA_STRING || c2 == NA_STRING)
 		LOGICAL(ans)[i] = NA_LOGICAL;
- 	    else if (c1 == c2)
-		LOGICAL(ans)[i] = 0;
- 	    else
-		if (Seql(c1, c2))
-		    LOGICAL(ans)[i] = 0;
-		else
-		    LOGICAL(ans)[i] = 1;
+	    else
+		LOGICAL(ans)[i] = Seql(c1, c2) ? 0 : 1;
 	}
 	break;
     case LTOP:
@@ -487,15 +477,15 @@ static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 	    c2 = STRING_ELT(s2, i % n2);
 	    if (c1 == NA_STRING || c2 == NA_STRING)
 		LOGICAL(ans)[i] = NA_LOGICAL;
- 	    else if (c1 == c2)
+	    else if (c1 == c2)
 		LOGICAL(ans)[i] = 0;
- 	    else {
+	    else {
 		errno = 0;
 		res = Scollate(c1, c2);
-		if(errno)  
+		if(errno)
 		    LOGICAL(ans)[i] = NA_LOGICAL;
 		else
-		    LOGICAL(ans)[i] = (res < 0)?1:0;
+		    LOGICAL(ans)[i] = (res < 0) ? 1 : 0;
 	    }
 	}
 	break;
@@ -505,15 +495,15 @@ static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 	    c2 = STRING_ELT(s2, i % n2);
 	    if (c1 == NA_STRING || c2 == NA_STRING)
 		LOGICAL(ans)[i] = NA_LOGICAL;
- 	    else if (c1 == c2)
+	    else if (c1 == c2)
 		LOGICAL(ans)[i] = 0;
- 	    else {
+	    else {
 		errno = 0;
 		res = Scollate(c1, c2);
-		if(errno)  
+		if(errno)
 		    LOGICAL(ans)[i] = NA_LOGICAL;
 		else
-		    LOGICAL(ans)[i] = (res > 0)?1:0;
+		    LOGICAL(ans)[i] = (res > 0) ? 1 : 0;
 	    }
 	}
 	break;
@@ -523,15 +513,15 @@ static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 	    c2 = STRING_ELT(s2, i % n2);
 	    if (c1 == NA_STRING || c2 == NA_STRING)
 		LOGICAL(ans)[i] = NA_LOGICAL;
- 	    else if (c1 == c2)
+	    else if (c1 == c2)
 		LOGICAL(ans)[i] = 1;
- 	    else {
+	    else {
 		errno = 0;
 		res = Scollate(STRING_ELT(s1, i % n1), STRING_ELT(s2, i % n2));
-		if(errno)  
+		if(errno)
 		    LOGICAL(ans)[i] = NA_LOGICAL;
 		else
-		    LOGICAL(ans)[i] = (res <= 0)?1:0;
+		    LOGICAL(ans)[i] = (res <= 0) ? 1 : 0;
 	    }
 	}
 	break;
@@ -541,20 +531,20 @@ static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 	    c2 = STRING_ELT(s2, i % n2);
 	    if (c1 == NA_STRING || c2 == NA_STRING)
 		LOGICAL(ans)[i] = NA_LOGICAL;
- 	    else if (c1 == c2)
+	    else if (c1 == c2)
 		LOGICAL(ans)[i] = 1;
-  	    else {
+	    else {
 		errno = 0;
 		res = Scollate(STRING_ELT(s1, i % n1), STRING_ELT(s2, i % n2));
-		if(errno)  
+		if(errno)
 		    LOGICAL(ans)[i] = NA_LOGICAL;
 		else
-		    LOGICAL(ans)[i] = (res >= 0)?1:0;
+		    LOGICAL(ans)[i] = (res >= 0) ? 1 : 0;
 	    }
 	}
 	break;
     }
-    UNPROTECT(2);
+    UNPROTECT(3);
     return ans;
 }
 
