@@ -192,8 +192,6 @@ static int sequal(SEXP x, int i, SEXP y, int j)
     /* Then if either is NA the other cannot be */
     if (STRING_ELT(x, i) == NA_STRING || STRING_ELT(y, j) == NA_STRING)
 	return 0;
-    /* Look at the lengths, as that is some check for embedded nuls. */
-    if (LENGTH(STRING_ELT(x, i)) != LENGTH(STRING_ELT(y, j))) return 0;
     /* Finally look at the contents if necessary */
     return !strcmp(translateChar(STRING_ELT(x, i)),
 		   translateChar(STRING_ELT(y, j)));
@@ -409,12 +407,11 @@ SEXP duplicated3(SEXP x, SEXP incomp, Rboolean from_last)
 {
     SEXP ans;
     int *h, *v;
-    int i, n;
+    int i, j, n,m;
     HashData data;
 
     if (!isVector(x))
 	error(_("'duplicated' applies only to vectors"));
-    PROTECT(incomp = coerceVector(incomp, TYPEOF(x)));
 
     n = LENGTH(x);
     HashTableSetup(x, &data);
@@ -425,14 +422,23 @@ SEXP duplicated3(SEXP x, SEXP incomp, Rboolean from_last)
     v = LOGICAL(ans);
 
     for (i = 0; i < data.M; i++) h[i] = NIL;
-    for (i = 0; i < length(incomp); i++) removeEntry(incomp, i, &data);
-    UNPROTECT(1);
 
     if(from_last)
 	for (i = n-1; i >= 0; i--) v[i] = isDuplicated(x, i, &data);
     else
 	for (i = 0; i < n; i++) v[i] = isDuplicated(x, i, &data);
 
+    if(length(incomp)) {
+	PROTECT(incomp = coerceVector(incomp, TYPEOF(x)));
+	m = length(incomp);
+	for (i = 0; i < n; i++) 
+	    if(v[i]) {
+		for(j = 0; j < m; j++)
+		    if(data.equal(x, i, incomp, j)) {v[i] = 0; break;}
+	    }
+	UNPROTECT(1);
+    }
+
     return ans;
 }
 
@@ -461,7 +467,7 @@ SEXP attribute_hidden do_duplicated(SEXP call, SEXP op, SEXP args, SEXP env)
     if(length(incomp) && /* S has FALSE to mean empty */
        !(isLogical(incomp) && length(incomp) == 1 && LOGICAL(incomp)[0] == 0))
 	dup = duplicated3(x, incomp, asLogical(CADDR(args)));
-    else 
+    else
 	dup = duplicated(x, asLogical(CADDR(args)));
 
     if (PRIMVAL(op) == 0) /* "duplicated()" : */
