@@ -477,8 +477,8 @@ all.equal(pgamma(x, 10, log = TRUE) - 10*log(x),
 all.equal(pgamma(x, 0.1, log = TRUE) - 0.1*log(x),
           rep(0.0498724412598364, length(x)), tol = 1e-13)# 7e-16 (i386)
 
-All.eq(dpois(  10, 2e-308, log=TRUE), -7100.13502718914)
-All.eq(dpois(  20, 3e-308, log=TRUE), -14204.2875435307)
+All.eq(dpois(  10*1:2, 3e-308, log=TRUE),
+       c(-7096.08037610806, -14204.2875435307))
 All.eq(dpois(1e20, 1e-290, log=TRUE), -7.12801378828154e+22)
 ## all gave -Inf in R 2.0.1
 
@@ -540,6 +540,15 @@ stopifnot(Inf == qgamma(1, sh))
 stopifnot(0   == qgamma(0, sh))
 ## the first gave Inf, NaN, and 99.425 in R 2.1.1 and earlier
 
+## In extreme left tail {PR#11030}
+qg <- qgamma(10:123*1e-12, shape=19)
+qg2<- qgamma(1:100 * 1e-9, shape=11)
+stopifnot(diff(qg, diff=2) < -6e-6,
+          diff(qg2,diff=2) < -6e-6,
+          All.eq(qg  [1], 2.35047385139143),
+          All.eq(qg2[30], 1.11512318734547))
+## was non-continuous in R 2.6.2 and earlier
+
 f2 <- c(0.5, 1:4)
 stopifnot(df(0, 1, f2) == Inf,
           df(0, 2, f2) == 1,
@@ -552,11 +561,15 @@ stopifnot(pbinom(x0, size = 3, prob = 0.1) == 0,
 ## very small negatives were rounded to 0 in R 2.2.1 and earlier
 
 ## dbeta(*, ncp):
+db.x <- c(0, 5, 80, 405, 1280, 3125, 6480, 12005, 20480, 32805,
+	  50000, 73205, 103680, 142805, 192080, 253125, 327680)
 a <- rlnorm(100)
 stopifnot(All.eq(a, dbeta(0, 1, a, ncp=0)),
-          dbeta(0, 0.9, 2.2, ncp = c(0, a)) == Inf
+	  dbeta(0, 0.9, 2.2, ncp = c(0, a)) == Inf,
+	  All.eq(65536 * dbeta(0:16/16, 5,1), db.x),
+	  All.eq(exp(16 * log(2) + dbeta(0:16/16, 5,1, log=TRUE)), db.x)
           )
-## the first gave 0, the 2nd NaN in R <= 2.3.0
+## the first gave 0, the 2nd NaN in R <= 2.3.0; others use 'TRUE' values
 
 ## df(*, ncp):
 x <- seq(0, 10, length=101)
@@ -610,6 +623,16 @@ for(nu in df.set) {
     stopifnot(is.finite(pqq))
 }
 
+All.eq(pt(2^-30, df=10),
+       0.50000000036238542)# = .5+ integrate(dt, 0,2^-30, df=10, rel.tol=1e-20)
+
+## rbinom(*, size) gave NaN for large size up to R <= 2.6.1
+M <- .Machine$integer.max
+set.seed(7)
+tt <- table(rbinom(100,    M, pr = 1e-9)) # had values in {0,2} only
+t2 <- table(rbinom(100, 10*M, pr = 1e-10))
+stopifnot(names(tt) == 0:6, sum(tt) == 100, sum(t2) == 100) ## no NaN there
+
 
 
 cat("Time elapsed: ", proc.time() - .ptime,"\n")
